@@ -35,6 +35,82 @@
 }
 
 
+"plot.acm"  <- function(x, xax = 1, yax = 2, pos = -1, storeData = TRUE, plot = TRUE, ...) {
+  if(!inherits(x, "dudi"))
+    stop("Object of class 'dudi' expected")
+  if(!inherits(x, "acm"))
+    stop("Object of class 'acm' expected")
+  
+  if((xax == yax) || (x$nf == 1))
+    stop("One axis only : not yet implemented")
+  if(length(xax) > 1 | length(yax) > 1)
+    stop("Not implemented for multiple xax/yax")
+  
+  if(xax > x$nf)
+    stop("Non convenient xax")
+  if(yax > x$nf)
+    stop("Non convenient yax")
+  
+  ## prepare
+  oritab <- as.list(x$call)[[2]]
+  
+  ## parameter management
+  sortparameters <- sortparamADEg(...)
+  params <- list()
+  params$g.args <- list(starSize = 0)
+  sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
+  
+  object <- do.call("s.class", c(list(dfxy = substitute(x$li), fac = oritab, xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$adepar, sortparameters$trellis, sortparameters$g.args, sortparameters$rest))
+  object@Call <- match.call()
+  if(plot)
+    print(object)
+  invisible(object)
+}
+
+
+"plot.fca" <- function(x, xax = 1, yax = 2, pos = -1, storeData = TRUE, plot = TRUE, ...) {
+  if(!inherits(x, "dudi"))
+    stop("Object of class 'dudi' expected")
+  if(!inherits(x, "fca"))
+    stop("Object of class 'fca' expected")
+  
+  if((xax == yax) || (x$nf == 1))
+    stop("One axis only : not yet implemented")
+  if(length(xax) > 1 | length(yax) > 1)
+    stop("Not implemented for multiple xax/yax")
+  
+  if(xax > x$nf)
+    stop("Non convenient xax")
+  if(yax > x$nf)
+    stop("Non convenient yax")
+  
+  ## prepare
+  oritab <- as.list(x$call)[[2]]
+  evTab <- eval.parent(oritab)
+  indica <- factor(rep(names(x$blo), x$blo))
+  ng <- length(levels(indica))
+  
+  ## parameter management
+  graphsnames <- as.character(levels(indica))
+  sortparameters <- sortparamADEgS(..., graphsnames = graphsnames)
+  params <- list()
+  params <- lapply(1:length(graphsnames), function(i) {params[[i]] <- list(starSize = 0.5, ellipseSize = 0, plabels = list(cex = 1.25), psub = list(text = graphsnames[i]))})
+  names(params) <- graphsnames
+  sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
+  
+  ## creation of each individual ADEg
+  l <- list()
+  l <- sapply(1:length(levels(indica)), function(i) {do.call("s.distri", c(list(dfxy = substitute(x$l1, env = sys.frame(-3)), dfdistri = call("[", oritab, call(":", 1, nrow(evTab)), which(indica == levels(indica)[i])), xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[i]]))})
+  
+  ## ADEgS creation
+  object <- new(Class = "ADEgS", ADEglist = l, positions = layout2position(.n2mfrow(ng), ng  = ng), add = matrix(0, ncol = ng, nrow = ng), Call = match.call())
+  names(object) <- graphsnames
+  if(plot)
+    print(object)
+  invisible(object)
+}
+
+
 "plot.coinertia" <- function(x, xax = 1, yax = 2, pos = -1, storeData = TRUE, plot = TRUE, ...) {
   if(!inherits(x, "coinertia")) 
     stop("Object of class 'coinertia' expected")
@@ -205,7 +281,6 @@
     stop("Non convenient yax")
   
   appel <- as.list(x$call)
-  fac <- eval.parent(appel$fac)
   
   ## sort parameters for each graph
   graphsnames <- c("Rrow", "Qrow", "Rax", "Rloadings", "Qloadings", "Qax", "eig")
@@ -256,8 +331,6 @@
     stop("Non convenient yax")
   
   appel <- as.list(x$call)
-  fac <- eval.parent(appel$fac)
-  dudi <- eval.parent(appel$x)
   
   ## sort parameters for each graph
   graphsnames <- c("loadings", "col", "eig", "row", "Xax", "class")
@@ -278,7 +351,7 @@
   g1 <- do.call("s.arrow", c(list(dfxy = substitute(x$c1), xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[1]]))
   g2 <- do.call("s.arrow", c(list(dfxy = substitute(x$co), xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[2]]))
   g3 <- do.call("plotEig", c(list(eigvalue = substitute(x$eig), nf = 1:x$nf, xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[3]])) 
-  g4 <- do.call("s.class", c(list(dfxy = substitute(x$ls), wt = substitute(dudi$lw), fac = appel$fac, xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[4]]))
+  g4 <- do.call("s.class", c(list(dfxy = substitute(x$ls), wt = call("$", appel$x, "lw"), fac = appel$fac, xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[4]]))
   g5 <- do.call("s.corcircle", c(list(dfxy = substitute(x$as), xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[5]]))
   g6 <- do.call("s.label", c(list(dfxy = substitute(x$li), xax = xax, yax = yax, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters[[6]]))
   
@@ -306,7 +379,6 @@
     stop("Non convenient yax")
   
   appel <- as.list(x$call)
-  fac <- eval.parent(appel$fac)
   
   ## sort parameters for each graph  
   graphsnames <- c("loadings", "col", "eig", "row", "Xax", "class")
@@ -355,7 +427,6 @@
     stop("Non convenient yax")
   
   appel <- as.list(x$call)
-  fac <- eval.parent(appel$fac)
   
   ## sort parameters for each graph
   graphsnames <- c("loadings", "col", "eig", "row", "Xax", "ccrow")
@@ -468,7 +539,6 @@
     stop("Non convenient yax")
   
   appel <- as.list(x$call)
-  fac <- eval.parent(appel$fac)
   
   ## sort parameters for each graph
   graphsnames <- c("Rrow", "Qrow", "Rax", "Rloadings", "Qloadings", "Qax", "eig")
@@ -1009,7 +1079,6 @@
     stop("Non convenient yax")
   
   appel <- as.list(x$call)
-  fac <- eval.parent(appel$fac)
   
   ## sort parameters for each graph
   graphsnames <- c("Rrow", "Qrow", "Rax", "Rloadings","Qloadings", "Qax", "eig")
@@ -1233,10 +1302,10 @@
   
   ## default values for parameters
   params <- list()
-  params[[1]] <- list(plines.col = "red", ppoints.col = "red", p1d.horizontal = FALSE, paxes.draw = TRUE, ppoints.cex = 2, ylab = "Root Mean Square Error", ylim = lim, porigin = origin)
-  params[[2]] <- list(plines.col = "red", ppoly.col = "red", p1d.horizontal = FALSE, paxes.draw = TRUE, method = "bars")
-  params[[3]] <- list(plines.col = "blue", ppoints.col = "blue", p1d.horizontal = FALSE, paxes.draw = TRUE, ppoints.cex = 2)
-  params[[4]] <- list(plines.col = "blue", ppoly.col = "blue", p1d.horizontal = FALSE, paxes.draw = TRUE, method = "bars")
+  params[[1]] <- list(plines = list(col = "red"), ppoints = list(col = "red", cex = 2), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), ylab = "Root Mean Square Error", ylim = lim, porigin = origin)
+  params[[2]] <- list(plines = list(col = "red"), ppolygons = list(col = "red"), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), method = "bars")
+  params[[3]] <- list(plines = list(col = "blue"), ppoints = list(col = "blue", cex = 2), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE))
+  params[[4]] <- list(plines = list(col = "blue"), ppolygons = list(col = "blue"), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), method = "bars")
   names(params) <- graphsnames
   sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
   
@@ -1272,10 +1341,10 @@
   
   ## default values for parameters
   params <- list()
-  params[[1]] <- list(plines.col = "red", ppoints.col = "red", p1d.horizontal = FALSE, paxes.draw = TRUE, ppoints.cex = 2, ylab = "Root Mean Square Error", ylim = lim, porigin = origin)
-  params[[2]] <- list(plines.col = "red", ppoly.col = "red", p1d.horizontal = FALSE, paxes.draw = TRUE, method = "area")
-  params[[3]] <- list(plines.col = "blue", ppoints.col = "blue", p1d.horizontal = FALSE, paxes.draw = TRUE, ppoints.cex = 2)
-  params[[4]] <- list(plines.col = "blue", ppoly.col = "blue", p1d.horizontal = FALSE, paxes.draw = TRUE, method = "area")
+  params[[1]] <- list(plines = list(col = "red"), ppoints = list(col = "red", cex = 2), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), ylab = "Root Mean Square Error", ylim = lim, porigin = origin)
+  params[[2]] <- list(plines = list(col = "red"), ppolygons = list(col = "red"), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), method = "area")
+  params[[3]] <- list(plines = list(col = "blue"), ppoints = list(col = "blue", cex = 2), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE))
+  params[[4]] <- list(plines = list(col = "blue"), ppolygons = list(col = "blue"), p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), method = "area")
   names(params) <- graphsnames
   sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
   
@@ -1311,8 +1380,8 @@
   
   ## default values for parameters
   params <- list()
-  params[[1]] <- list(p1d.horizontal = FALSE, paxes.draw = TRUE, ppoints.cex = 2, ylim = lim, porigin = origin)
-  params[[2]] <- list(p1d.horizontal = FALSE, paxes.draw = TRUE, method = "bars")
+  params[[1]] <- list(p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), ppoints = list(cex = 2), ylim = lim, porigin = origin)
+  params[[2]] <- list(p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), method = "bars")
   names(params) <- graphsnames
   sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
   
@@ -1345,8 +1414,8 @@
   
   ## default values for parameters
   params <- list()
-  params[[1]] <- list(p1d.horizontal = FALSE, paxes.draw = TRUE, ppoints.cex = 2, ylim = lim, porigin = origin)
-  params[[2]] <- list(p1d.horizontal = FALSE, paxes.draw = TRUE, method = "bars")
+  params[[1]] <- list(p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), ppoints = list(cex = 2), ylim = lim, porigin = origin)
+  params[[2]] <- list(p1d = list(horizontal = FALSE), paxes = list(draw = TRUE), method = "bars")
   names(params) <- graphsnames
   sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
   
@@ -1367,7 +1436,7 @@
 
 
 "plot.inertia" <- function(x, xax = 1, yax = 2, cont = 0.1, type = c("label", "cross", "ellipse", "both"), ellipseSize = 1.5, 
-                           posieig = "none", plot = TRUE, storeData = TRUE, pos = -1, ...) { 
+                           posieig = "none", plot = TRUE, storeData = TRUE, pos = -1, ...) {
   
   if(!inherits(x, "inertia")) 
     stop("Object of class 'inertia' expected")
@@ -1384,11 +1453,12 @@
   if(length(yax) > 1)
     stop("Not implemented for multiple yax")
   if(yax > evTab$nf)
-    stop("Non convenient yax") 
+    stop("Non convenient yax")
   
   adegtot <- adegpar()
   position <- match.arg(posieig[1], choices = c("bottomleft", "bottomright", "topleft", "topright", "none"), several.ok = FALSE)
   type <- match.arg(type)
+  type <- type[1]
   
   ## sort parameters for each graph
   graphsnames <- c("light_row", "heavy_row", "light_col", "heavy_col", "eig")
@@ -1401,30 +1471,35 @@
   params$light_col <- list(plabels = list(cex = 0), ppoints = list(col = "grey20", alpha = 0.45, cex = 1.2, pch = 19))
   
   if(type == "label") {
-    params$heavy_row <- list(plabels = list(box = list(draw = FALSE), col = "red"), ppoints = list(cex = 0))
-    params$heavy_col <- list(plabels = list(box = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0))
+    params$heavy_row <- list(plabels = list(boxes = list(draw = FALSE), col = "red"), ppoints = list(cex = 0))
+    params$heavy_col <- list(plabels = list(boxes = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0))
   } else if(type == "cross") {
-    params$heavy_row <- list(ellipseSize = ellipseSize, plabels = list(box = list(draw = FALSE), col = "red"), ppoints = list(cex = 0), pellipses = list(lwd = 0, axes = list(col = "red", lty = 1)), plines = list(lwd = 0), plegend = list(drawKey = FALSE))
-    params$heavy_col <- list(ellipseSize = ellipseSize, plabels = list(box = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0), pellipses = list(lwd = 0, axes = list(col = "blue", lty = 1)), plines = list(lwd = 0), plegend = list(drawKey = FALSE))
+    params$heavy_row <- list(ellipseSize = ellipseSize, plabels = list(boxes = list(draw = FALSE), col = "red"), ppoints = list(cex = 0), pellipses = list(lwd = 0, axes = list(col = "red", lty = 1)), plines = list(lwd = 0), plegend = list(drawKey = FALSE))
+    params$heavy_col <- list(ellipseSize = ellipseSize, plabels = list(boxes = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0), pellipses = list(lwd = 0, axes = list(col = "blue", lty = 1)), plines = list(lwd = 0), plegend = list(drawKey = FALSE))
   } else if(type == "ellipse") {
-    params$heavy_row <- list(ellipseSize = ellipseSize, plabels = list(box = list(draw = FALSE), col = "red"), ppoints = list(cex = 0), pellipses = list(border = "red", axes = list(lwd = 0)), plines = list(col = 0))
-    params$heavy_col <- list(ellipseSize = ellipseSize, plabels = list(box = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0), pellipses = list(border = "blue", axes = list(lwd = 0)), plines = list(lwd = 0))
+    params$heavy_row <- list(ellipseSize = ellipseSize, plabels = list(boxes = list(draw = FALSE), col = "red"), ppoints = list(cex = 0), pellipses = list(border = "red", axes = list(lwd = 0)), plines = list(col = 0))
+    params$heavy_col <- list(ellipseSize = ellipseSize, plabels = list(boxes = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0), pellipses = list(border = "blue", axes = list(lwd = 0)), plines = list(lwd = 0))
   } else if(type == "both") {
-    params$heavy_row <- list(ellipseSize = 1.5, plabels = list(box = list(draw = FALSE), col = "red"), ppoints = list(cex = 0), pellipses = list(border = "red", axes = list(col = "red", lty = 1)), plines = list(lwd = 0))
-    params$heavy_col <- list(ellipseSize = 1.5, plabels = list(box = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0), pellipses = list(border = "blue", axes = list(col = "blue", lty = 1)), plines = list(lwd = 0))
+    params$heavy_row <- list(ellipseSize = 1.5, plabels = list(boxes = list(draw = FALSE), col = "red"), ppoints = list(cex = 0), pellipses = list(border = "red", axes = list(col = "red", lty = 1)), plines = list(lwd = 0))
+    params$heavy_col <- list(ellipseSize = 1.5, plabels = list(boxes = list(draw = FALSE), col = "blue"), ppoints = list(cex = 0), pellipses = list(border = "blue", axes = list(col = "blue", lty = 1)), plines = list(lwd = 0))
   }
   params$eig <- list(pbackground = list(box = TRUE), psub = list(text = "Eigenvalues"))
   sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
-
+  
+  # never display points under contribution threshold
+  sortparameters$light_row$plabels$cex <- 0
+  sortparameters$light_col$plabels$cex <- 0
+  
   ## management of the data and the parameters about the rows' contribution (individuals) on axes
-  if(!is.null(x$row.abs)) {
-    inertrow <- x$row.abs[, c(xax, yax)] / 100
-    inertrowcall <- call("/", call("[", call("$", substitute(x), "row.abs"), call(":", 1, call("NROW", call("$", substitute(x), "row.abs"))), c(xax, yax)), 100)
-    light_row <- subset(evTab$li[, c(xax, yax)], inertrow[, 1] < cont & inertrow[, 2] < cont)
-    light_rowcall <- call("subset", call("[", call("$", ori[[2]], "li"), call(":", 1, call("NROW", call("$", ori[[2]], "li"))), c(xax, yax)), call("&", call("<", call("[", inertrowcall, 1), cont), call("<", call("[", inertrowcall, 2), cont)))
+  if(!is.null(x$row.rel)) {
+    inertrow <- abs(x$row.rel[, c(xax, yax)]) / 100
+    inertrowcall <- call("/", call("abs", call("[", call("$", substitute(x), "row.rel"), call(":", 1, call("NROW", call("$", substitute(x), "row.rel"))), c(xax, yax))), 100)
+    lightrow <- subset(evTab$li[, c(xax, yax)], inertrow[, 1] < cont & inertrow[, 2] < cont)
+    lightrowcall <- call("subset", call("[", call("$", ori[[2]], "li"), call(":", 1, call("NROW", call("$", ori[[2]], "li"))), c(xax, yax)), call("&", call("<", call("[", inertrowcall, 1), cont), call("<", call("[", inertrowcall, 2), cont)))
     
-    heavy_row <- subset(evTab$li[, c(xax, yax)], inertrow[, 1] >= cont | inertrow[, 2] >= cont)
-    if(nrow(heavy_row) == 0)
+    heavyrow <- subset(evTab$li[, c(xax, yax)], inertrow[, 1] >= cont | inertrow[, 2] >= cont)
+    heavyrowcall <- call("subset", call("[", call("$", ori[[2]], "li"), call(":", 1, call("NROW", call("$", ori[[2]], "li"))), c(xax, yax)), call("&", call(">=", call("[", inertrowcall, 1), cont), call(">=", call("[", inertrowcall, 2), cont)))
+    if(nrow(heavyrow) == 0)
       stop("No points to draw, try lowering 'cont'")
     heavy_inertrow <- subset(inertrow, inertrow[, 1] >= cont | inertrow[, 2] >= cont)
     heavy_inertrowcum <- apply(heavy_inertrow, 1, sum)
@@ -1437,40 +1512,41 @@
       }
     }
     
-    lim.global <- setlimits2D(minX = min(c(heavy_row[, 1], light_row[, 1])), maxX = max(c(heavy_row[, 1], light_row[, 1])), 
-                              minY = min(c(heavy_row[, 2], light_row[, 2])), maxY = max(c(heavy_row[, 2], light_row[, 2])),
-                              origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
+    limglobal <- setlimits2D(minX = min(c(heavyrow[, 1], lightrow[, 1])), maxX = max(c(heavyrow[, 1], lightrow[, 1])), 
+                             minY = min(c(heavyrow[, 2], lightrow[, 2])), maxY = max(c(heavyrow[, 2], lightrow[, 2])),
+                             origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
     
     if(type != "label") {
       # if ellipses or crosses are drawn, the limits are re-calculated and the elipses size are normalized
       heavy_inertrowmax <- apply(heavy_inertrow, 2, max)
       heavy_inertrownorm <- matrix(NA, NROW(heavy_inertrow), 2)
-      for (i in 1:2) {heavy_inertrownorm[, i] <- (heavy_inertrow[, i] / heavy_inertrowmax[i]) * (diff(lim.global[[i]]) / 10)}
+      for (i in 1:2) {heavy_inertrownorm[, i] <- (heavy_inertrow[, i] / heavy_inertrowmax[i]) * (diff(limglobal[[i]]) / 10)}
       
       # TODO
       # add 0.00001 to the coordinates to avoid the bug in the '.util.ellipse' function (waiting to correct it)
-      cont_row <- cbind(c(heavy_row[, 1] - heavy_inertrownorm[, 1]/2, heavy_row[, 1] + heavy_inertrownorm[, 1]/2, heavy_row[, 1], heavy_row[, 1] + 0.00001), 
-                        c(heavy_row[, 2] + 0.00001, heavy_row[, 2], heavy_row[, 2] - heavy_inertrownorm[, 2]/2, heavy_row[, 2] + heavy_inertrownorm[, 2]/2)) 
-      fac_row <- as.factor(rep(rownames(heavy_row), 4))
-      lim.global <- setlimits2D(minX = min(c(cont_row[, 1], light_row[, 1])), maxX = max(c(cont_row[, 1], light_row[, 1])), 
-                                minY = min(c(cont_row[, 2], light_row[, 2])), maxY = max(c(cont_row[, 2], light_row[, 2])),
-                                origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
+      cont_row <- cbind(c(heavyrow[, 1] - heavy_inertrownorm[, 1]/2, heavyrow[, 1] + heavy_inertrownorm[, 1]/2, heavyrow[, 1], heavyrow[, 1] + 0.00001), 
+                        c(heavyrow[, 2] + 0.00001, heavyrow[, 2], heavyrow[, 2] - heavy_inertrownorm[, 2]/2, heavyrow[, 2] + heavy_inertrownorm[, 2]/2)) 
+      fac_row <- as.factor(rep(rownames(heavyrow), 4))
+      limglobal <- setlimits2D(minX = min(c(cont_row[, 1], lightrow[, 1])), maxX = max(c(cont_row[, 1], lightrow[, 1])), 
+                               minY = min(c(cont_row[, 2], lightrow[, 2])), maxY = max(c(cont_row[, 2], lightrow[, 2])),
+                               origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
     }
     
     params <- list()
-    params$light_row <- list(xlim = lim.global$xlim, ylim = lim.global$ylim)
+    params$light_row <- list(xlim = limglobal$xlim, ylim = limglobal$ylim)
     sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
   }
   
   ## management of the data and the parameters about the columns' contribution (variables) on axes
-  if(!is.null(x$col.abs)) {
-    inertcol <- x$col.abs[, c(xax, yax)] / 100
-    inertcolcall <- call("/", call("[", call("$", substitute(x), "col.abs"), call(":", 1, call("NROW", call("$", substitute(x), "col.abs"))), c(xax, yax)), 100)
-    light_col <- subset(evTab$co[, c(xax, yax)], inertcol[, 1] < cont & inertcol[, 2] < cont)
-    light_colcall <- call("subset", call("[", call("$", ori[[2]], "co"), call(":", 1, call("NROW", call("$", ori[[2]], "co"))), c(xax, yax)), call("&", call("<", call("[", inertcolcall, 1), cont), call("<", call("[", inertcolcall, 2), cont)))
+  if(!is.null(x$col.rel)) {
+    inertcol <- abs(x$col.rel[, c(xax, yax)]) / 100
+    inertcolcall <- call("/", call("abs", call("[", call("$", substitute(x), "col.rel"), call(":", 1, call("NROW", call("$", substitute(x), "col.rel"))), c(xax, yax))), 100)
+    lightcol <- subset(evTab$co[, c(xax, yax)], inertcol[, 1] < cont & inertcol[, 2] < cont)
+    lightcolcall <- call("subset", call("[", call("$", ori[[2]], "co"), call(":", 1, call("NROW", call("$", ori[[2]], "co"))), c(xax, yax)), call("&", call("<", call("[", inertcolcall, 1), cont), call("<", call("[", inertcolcall, 2), cont)))
     
-    heavy_col <- subset(evTab$co[, c(xax, yax)], inertcol[, 1] >= cont | inertcol[, 2] >= cont)
-    if(nrow(heavy_col) == 0)
+    heavycol <- subset(evTab$co[, c(xax, yax)], inertcol[, 1] >= cont | inertcol[, 2] >= cont)
+    heavycolcall <- call("subset", call("[", call("$", ori[[2]], "co"), call(":", 1, call("NROW", call("$", ori[[2]], "co"))), c(xax, yax)), call("&", call(">=", call("[", inertcolcall, 1), cont), call(">=", call("[", inertcolcall, 2), cont)))
+    if(nrow(heavycol) == 0)
       stop("No points to draw, try lowering 'cont'")
     heavy_inertcol <- subset(inertcol, inertcol[, 1] >= cont | inertcol[, 2] >= cont)
     heavy_inertcolcum <- apply(heavy_inertcol, 1, sum)
@@ -1483,28 +1559,28 @@
       }
     }
     
-    lim.global <- setlimits2D(minX = min(c(heavy_col[, 1], light_col[, 1])), maxX = max(c(heavy_col[, 1], light_col[, 1])), 
-                              minY = min(c(heavy_col[, 2], light_col[, 2])), maxY = max(c(heavy_col[, 2], light_col[, 2])),
-                              origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
+    limglobal <- setlimits2D(minX = min(c(heavycol[, 1], lightcol[, 1])), maxX = max(c(heavycol[, 1], lightcol[, 1])), 
+                             minY = min(c(heavycol[, 2], lightcol[, 2])), maxY = max(c(heavycol[, 2], lightcol[, 2])),
+                             origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
     
     if(type != "label") {
       # if ellipses or crosses are drawn, the limits are re-calculated and the ellipse size are normalized
       heavy_inertcolmax <- apply(heavy_inertcol, 2, max)
       heavy_inertcolnorm <- matrix(NA, NROW(heavy_inertcol), 2)
-      for (i in 1:2) {heavy_inertcolnorm[, i] <- (heavy_inertcol[, i] / heavy_inertcolmax[i]) * (diff(lim.global[[i]]) / 10)}
+      for (i in 1:2) {heavy_inertcolnorm[, i] <- (heavy_inertcol[, i] / heavy_inertcolmax[i]) * (diff(limglobal[[i]]) / 10)}
       
       # TODO
       # add 0.00001 to the coordinates to avoid the bug in the '.util.ellipse' function (waiting to correct it)
-      cont_col <- cbind(c(heavy_col[, 1] - heavy_inertcolnorm[, 1]/2, heavy_col[, 1] + heavy_inertcolnorm[, 1]/2, heavy_col[, 1], heavy_col[, 1] + 0.00001), 
-                        c(heavy_col[, 2] + 0.00001, heavy_col[, 2], heavy_col[, 2] - heavy_inertcolnorm[, 2]/2, heavy_col[, 2] + heavy_inertcolnorm[, 2]/2))  
-      fac_col <- as.factor(rep(rownames(heavy_col), 4))
-      lim.global <- setlimits2D(minX = min(c(cont_col[, 1], light_col[, 1])), maxX = max(c(cont_col[, 1], light_col[, 1])), 
-                                minY = min(c(cont_col[, 2], light_col[, 2])), maxY = max(c(cont_col[, 2], light_col[, 2])),
-                                origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
+      cont_col <- cbind(c(heavycol[, 1] - heavy_inertcolnorm[, 1]/2, heavycol[, 1] + heavy_inertcolnorm[, 1]/2, heavycol[, 1], heavycol[, 1] + 0.00001), 
+                        c(heavycol[, 2] + 0.00001, heavycol[, 2], heavycol[, 2] - heavy_inertcolnorm[, 2]/2, heavycol[, 2] + heavy_inertcolnorm[, 2]/2))  
+      fac_col <- as.factor(rep(rownames(heavycol), 4))
+      limglobal <- setlimits2D(minX = min(c(cont_col[, 1], lightcol[, 1])), maxX = max(c(cont_col[, 1], lightcol[, 1])), 
+                               minY = min(c(cont_col[, 2], lightcol[, 2])), maxY = max(c(cont_col[, 2], lightcol[, 2])),
+                               origin = adegtot$porigin$origin, aspect.ratio = adegtot$paxes$aspectratio, includeOr = adegtot$porigin$include)
     }
     
     params <- list()
-    params$light_col <- list(xlim = lim.global$xlim, ylim = lim.global$ylim)
+    params$light_col <- list(xlim = limglobal$xlim, ylim = limglobal$ylim)
     sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
   }
   
@@ -1515,14 +1591,16 @@
   
   ## function to create the graphics about the row' contribution (individuals) on axes
   f_row <- function(posi = "none", pos){
-    graphnames <- c("light_row", "heavy_row", if(posi != "none") {"eig"})
-    g1 <- do.call("s.label", c(list(dfxy = light_row, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$light_row))
+    graphnames <- c(if(length(lightrow) > 0) {"light_row"}, "heavy_row", if(posi != "none") {"eig"})
+    
+    g1 <- do.call("s.label", c(list(dfxy = lightrow, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$light_row))
     if(type == "label")
-      g2 <- do.call("s.label", c(list(dfxy = heavy_row, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$heavy_row))
+      g2 <- do.call("s.label", c(list(dfxy = heavyrow, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$heavy_row))
     else
       g2 <- do.call("s.class", c(list(dfxy = cont_row, fac = fac_row, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$heavy_row))
     grow <- do.call("superpose", list(g1, g2))
     grow@Call <- call("superpose", list(g1@Call, g2@Call))
+    
     if(posi != "none")
       grow <- do.call("insert", list(geig, grow, posi = posi, plot = FALSE, ratio = 0.25))
     names(grow) <- graphnames
@@ -1531,14 +1609,16 @@
   
   # function to create the graphics about the columns' contribution (variables) on axes
   f_col <- function(posi = "none", pos) {
-    graphnames <- c("light_col", "heavy_col", if(posi != "none") {"eig"})
-    g3 <- do.call("s.label", c(list(dfxy = light_col, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$light_col))
+    graphnames <- c(if(length(lightcol) > 0) {"light_col"}, "heavy_col", if(posi != "none") {"eig"})
+    
+    g3 <- do.call("s.label", c(list(dfxy = lightcol, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$light_col))
     if(type == "label")
-      g4 <- do.call("s.label", c(list(dfxy = heavy_col, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$heavy_col))
+      g4 <- do.call("s.label", c(list(dfxy = heavycol, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$heavy_col))
     else
       g4 <- do.call("s.class", c(list(dfxy = cont_col, fac = fac_col, xax = 1, yax = 2, plot = FALSE, storeData = storeData, pos = pos - 2), sortparameters$heavy_col))
     gcol <- do.call("superpose", list(g3, g4))
     gcol@Call <- call("superpose", list(g3@Call, g4@Call))
+    
     if(posi != "none")
       gcol <- do.call("insert", list(geig, gcol, posi = posi, plot = FALSE, ratio = 0.25))
     names(gcol) <- graphnames
@@ -1553,13 +1633,13 @@
   }
   
   ## creation of the appropriate plot according to the input data
-  if(!is.null(x$row.abs) & is.null(x$col.abs))
+  if(!is.null(x$row.rel) & is.null(x$col.rel))
     object <- f_row(posi = position, pos = pos)
-  if(!is.null(x$col.abs) & is.null(x$row.abs))
+  if(!is.null(x$col.rel) & is.null(x$row.rel))
     object <- f_col(posi = position, pos = pos)
-  if(!is.null(x$row.abs) & !is.null(x$col.abs))
+  if(!is.null(x$row.rel) & !is.null(x$col.rel))
     object <- f_both(posi = position, pos = pos)
-  if(is.null(x$row.abs) & is.null(x$col.abs))
+  if(is.null(x$row.rel) & is.null(x$col.rel))
     stop(paste("No inertia was calculated in the ", substitute(x), " object", sep = ""))
   
   object@Call <- match.call()
@@ -1568,3 +1648,83 @@
     print(object)
   invisible(object)
 }
+
+
+"plot.randtest" <- function(x, nclass = 10, coeff = 1, pos = -1, storeData = TRUE, plot = TRUE, ...) {
+  
+  if(!inherits(x, "randtest")) 
+    stop("Object of class 'randtest' expected")
+  
+  # by default, in ade4, as.randtest computes the histogram with 10 class
+  # x$sim is available only if !inherits(x, "lightrandtest")
+  if(!inherits(x, "lightrandtest") & nclass != 10){
+    h0 <- hist(x$sim, plot = FALSE, nclass = nclass)
+  } else {
+    h0 <- x$plot$hist
+  }
+  
+  ## common limits
+  mylim <- x$plot$xlim
+  
+  ## parameter management
+  graphsnames <- c("sim", "obs")
+  sortparameters <- sortparamADEgS(..., graphsnames = graphsnames)
+  params <- list()
+  params[[1]] <- list(p1d = list(horizontal = TRUE), pgrid = list(draw = FALSE), paxes = list(draw = TRUE), xlim = mylim, main = "Histogram of sim", xlab = "sim", ylab = "Frequency")
+  params[[2]] <- list(plines = list(lwd = 1.5), ppoints = list(pch = 18, cex = 1.5))
+  names(params) <- graphsnames
+  sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
+  
+  ## plot creation
+  object <- plotRandTest(hist = h0, nclass = nclass, obs = x$obs, params = sortparameters)
+  names(object) <- graphsnames
+  object@Call <- match.call()
+  if(plot)
+    print(object)
+  invisible(object)
+}
+
+
+"plot.krandtest" <- function (x, nclass = 10, pos = -1, storeData = TRUE, plot = TRUE, ...) {
+  
+  if (!inherits(x, "krandtest")) 
+    stop("Object of class 'krandtest' expected")
+  
+  ng <- x$ntest
+  maintitle <- x$names
+  
+  ## parameter management
+  graphsnames <- paste0("g", seq_len(ng))
+  sortparameters <- sortparamADEgS(..., graphsnames = graphsnames)
+  params <- list()
+  params <- lapply(seq_len(ng), function(i) {params[[i]] <- list(p1d = list(horizontal = TRUE), pgrid = list(draw = FALSE), paxes = list(draw = TRUE), 
+                                                                           xlim = x$plot[[i]]$xlim, main = maintitle[i], xlab = "", ylab = "",
+                                                                           plines = list(lwd = 1.5), ppoints = list(pch = 18, cex = 1.5))})
+  names(params) <- graphsnames
+  sortparameters <- modifyList(params, sortparameters, keep.null = TRUE)
+  
+  
+  if(inherits(x, "lightkrandtest")) {
+    l <- list()
+    l <- sapply(seq_len(ng), function(i) {do.call("plotRandTest", c(list(hist = x$plot[[i]]$hist, nclass = nclass, obs = x$obs[i], params = sortparameters[[i]])))})
+    
+    ## ADEgS creation
+    object <- new(Class = "ADEgS", ADEglist = l, positions = layout2position(rev(.n2mfrow(ng)), ng  = ng), add = matrix(0, ncol = ng, nrow = ng), Call = match.call())
+    names(object) <- graphsnames
+    
+  } else {
+    l <- list()
+    for (k in 1:x$ntest) {
+      rd <- as.randtest(x$sim[, k], x$obs[k], output = "full")
+      l[[k]] <- do.call("plot.randtest", c(list(rd, nclass = nclass, plot = FALSE), sortparameters[[k]]))
+    }
+    ## ADEgS creation
+    object <- new(Class = "ADEgS", ADEglist = l, positions = layout2position(rev(.n2mfrow(ng)), ng  = ng), add = matrix(0, ncol = ng, nrow = ng), Call = match.call())
+    names(object) <- graphsnames
+  }
+  
+  object@Call <- match.call()
+  if(plot)
+    print(object)
+  invisible(object)
+}
